#!/bin/sh
#This is supposed to be a generic wrapper for sensors
#on various nodes to run the sensors and return the fan speeds 
#to ngop and ganglia
#We will store hardware type in /etc/hwtype
#That's an executable shell script that sets the levels
#of acceptable fan speed and the strings to grep for to get them.
# Right now it only supports up to six fans, which is the most 
# any of our nodes have, but it can be expanded later if necessary.
#
# With no arguments it will return a list of all fan speeds, space separated
# With argument -l (lower case l) it will return integer severity
# -2--fan is not supposed to be there  (ngop green)
# -1--data not available               (ngop gray)
# 0--normal                            (ngop green)
# 1--jammed sensors, if we can detect it  (ngop blue alarm+event)
# 2--slow fan, first occurence            (ngop yellow alarm)
# 3--off fan, first occurence             (ngop red alarm)
# 4--slow fan, persistent                 (ngop yellow event+alarm)
# 5--off fan, persistent                  (ngop red event+alarm)
# ngop will only use this integer output.
#
#Fan labels change in this release so that the same fan 
#always corresponds to the same number in the output.
# 1=cpufan1,2=cpufan2, 3=chassis fan 1, 4=chassis fan 2, 5=chassis fan 3,
# 6=chassis fan 4 (not implemented anywhere but may be eventually)
# using option -n 1 will return speed for fan 1
# using -l -n 1 will return logical result just for fan 1.
# fans not supposed to be there will return -2 for logical and numeric speed
# L440GX doesn't have any fans we can read out.
# Also no readable fans on the only Tyan 2723 nodes we have.
if [ -x /etc/hwtype ]
then
    . /etc/hwtype
else
    HWTYPE="L440GX"
    NFAN=0
fi 
# Look for the node-specific temperature file
if [ -x /etc/hwtype.node ]
then
    . /etc/hwtype.node
fi
#
#  Check history files--make sure if they exist they were there
#  at least three minutes
#
if [ -s /var/log/fan1_slow ]
then
    F1S="`find /var/log/fan1_slow -mmin +3 -print`"
    if [ "$F1S" != "" ] 
    then
	FAN1_SLOW="YES"
    fi
fi
if [ -s /var/log/fan2_slow ]
then
    F2S="`find /var/log/fan2_slow -mmin +3 -print`"
    if [ "$F2S" != "" ] 
    then
	FAN2_SLOW="YES"
    fi
fi
if [ -s /var/log/fan3_slow ]
then
    F3S="`find /var/log/fan3_slow -mmin +3 -print`"
    if [ "$F3S" != "" ] 
    then
	FAN3_SLOW="YES"
    fi
fi
if [ -s /var/log/fan4_slow ]
then
    F4S="`find /var/log/fan4_slow -mmin +3 -print`"
    if [ "$F4S" != "" ] 
    then
	FAN4_SLOW="YES"
    fi
fi
if [ -s /var/log/fan5_slow ]
then
    F5S="`find /var/log/fan5_slow -mmin +3 -print`"
    if [ "$F5S" != "" ] 
    then
	FAN5_SLOW="YES"
    fi
fi
if [ -s /var/log/fan6_slow ]
then
    F6S="`find /var/log/fan6_slow -mmin +3 -print`"
    if [ "$F6S" != "" ] 
    then
	FAN6_SLOW="YES"
    fi
fi
if [ -s /var/log/fan1_off ]
then
    F1O="`find /var/log/fan1_off -mmin +3 -print`"
    if [ "$F1O" != "" ] 
    then
	FAN1_OFF="YES"
    fi
fi
if [ -s /var/log/fan2_off ]
then
    F2O="`find /var/log/fan2_off -mmin +3 -print`"
    if [ "$F2O" != "" ] 
    then
	FAN2_OFF="YES"
    fi
fi
if [ -s /var/log/fan3_off ]
then
    F3O="`find /var/log/fan3_off -mmin +3 -print`"
    if [ "$F3O" != "" ] 
    then
	FAN3_OFF="YES"
    fi
fi
if [ -s /var/log/fan4_off ]
then
    F4O="`find /var/log/fan4_off -mmin +3 -print`"
    if [ "$F4O" != "" ] 
    then
	FAN4_OFF="YES"
    fi
fi
if [ -s /var/log/fan5_off ]
then
    F5O="`find /var/log/fan5_off -mmin +3 -print`"
    if [ "$F5O" != "" ] 
    then
	FAN5_OFF="YES"
    fi
fi
if [ -s /var/log/fan6_off ]
then
    F6O="`find /var/log/fan6_off -mmin +3 -print`"
    if [ "$F6O" != "" ] 
    then
	FAN6_OFF="YES" 
    fi
fi
if [ -s /var/log/sensors.jammed ]
then
    T2JAM="`find /var/log/sensors.jammed -mmin +3 -print`"
    if [ "$T2JAM" != "" ]
    then
	SENSORS_JAMMED="YES"
    fi
fi
LOGRES="NUMERIC"
JAMMED="NO"
FAN_LOG=0
FAN1_LOG=-2
FAN2_LOG=-2
FAN3_LOG=-2
FAN4_LOG=-2
FAN5_LOG=-2
FAN6_LOG=-2
FAN1_VAL=-2
FAN2_VAL=-2
FAN3_VAL=-2
FAN4_VAL=-2
FAN5_VAL=-2
FAN6_VAL=-2
#locations of Fans in the reading, this only applies to DELL HWTYPE
FAN1_LOC=6
FAN2_LOC=7
FAN3_LOC=8
FAN4_LOC=9
FAN5_LOC=10
FAN6_LOC=11
NF=0
ALLFAN="NO"
while [ $# -ne 0 ] 
do
    case $1 in 
	-l) LOGRES="LOGICAL" ; shift 1 ;;
        -n) test $# -lt 2 && { echo "invalid arguments to -n" ; exit 1 ; }
	    NF=$2 ; shift 2 ;;
	-a) ALLFAN="YES" ; shift 1 ;;
        *)  echo "invalid arguments" ; exit 1 ;;
    esac
done
#Remove any sensors.out files older than 3 minutes old
#leave them there if they are younger than 3 minutes.
#This allows us to run sensors just once for multiple
#calls of sensors_wrap.sh and fan_wrap.sh by the ngop agent
if [ -s /tmp/sensors.out ]
then
    find /tmp/sensors.out -mmin +3 -exec rm -f '{}' ';'
fi
# Run sensors again only if we need to.
if [ ! -s /tmp/sensors.out ] 
then
#
#Check to see if sensors is already running (if so it's an error condition
# and the process is probably stuck
    NSENSORS=`ps -ef | grep /usr/bin/sensors | grep -v grep | wc -l`
    NOMREPORT=`ps -ef | grep /usr/bin/omreport | grep -v grep | wc -l`
    if [ $NSENSORS -eq 0 ] && [ $NOMREPORT -eq 0 ]
    then
	if [ $NFAN -gt 0 ]
	then
	    if [ "$HWTYPE" == "DELL" ] 
	    then
		if [ -x /usr/bin/omreport ] 
		then
		    /usr/bin/omreport chassis temps > /tmp/sensors.out 2>/tmp/sensors.err ; /usr/bin/omreport chassis fans >> /tmp/sensors.out 2>>/tmp/sensors.err
                fi
            else
		if [ -x /usr/bin/sensors ]
		then
		    /usr/bin/sensors > /tmp/sensors.out 2> /tmp/sensors.err
		fi
            fi
	else
#  By returning -2 in case of 2723 and L440GX ngop agent will show
#  green instead of grey.
	    if [ "$HWTYPE" == "2723" ] || [ "$HWTYPE" == "L440GX" ]
	    then
		FAN_LOG=-2
		FAN_LOGSEL=-2
		FANSTRING=""
		FAN_SEL=""
	    else
		FAN_LOG=-1
		FAN_LOGSEL=-1
		FANSTRING=""
		FAN_SEL=""
            fi
	fi
    else
# We get here in the case of a 2466 machine with stuck sensors
# where a /usr/bin/sensors is running and /tmp/sensors.out exists
# but is zero size, waiting 10 minutes to get filled up.
	if [ "$HWTYPE" == "2466" ]
	then
#Show the sensors as jammed
	    FAN_LOG=-1
	    FAN_LOGSEL=-1
	    FAN1_LOG=-1
	    FAN2_LOG=-1
	    FAN3_LOG=-1
	    FAN4_LOG=-1
	    FAN5_LOG=-1
	    FANSTRING=""
	    FAN_SEL=""
	    JAMMED="YES"
	else
#Show the sensors as unavailable
	    FAN_LOG=-1
	    FAN_LOGSEL=-1
	    FANSTRING=""
	    FAN_SEL=""
        fi
    fi
fi
if [ -s /tmp/sensors.out ] 
then
    case "$HWTYPE" in
    370DLE)
# The two fans here are both CPU fans.
    FAN1_VAL="`grep ${FAN1_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN2_VAL="`grep ${FAN2_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FANST0="$FAN1_VAL $FAN2_VAL"
#   Assume these are good unless we find out later they are bad
#   and set them that way
    FAN1_LOG=0
    FAN2_LOG=0
    ;;
    2518) 
#1U 2518--3 fan readings but two of them (cpu fans) are zero
#because there is no cpu fan..only fan is a chassis blower
#2U 2518--3 fan readings, 2 cpu fans and 1 chassis fan speed, common
#to all fans in center rack.
    if [ $NFAN -gt 1 ] 
    then
    FAN1_VAL="`grep ${FAN1_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN2_VAL="`grep ${FAN2_TEXT} /tmp/sensors.out | awk '{print $2}'`" 
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FANST0="$FAN1_VAL $FAN2_VAL $FAN3_VAL"
    FAN1_LOG=0
    FAN2_LOG=0
    FAN3_LOG=0
    else
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FANST0="$FAN3_VAL"
    FAN3_LOG=0
    fi
    ;;
    2468)
#   3 fans on 1U Tyan 2468, 2 cpu fans, one blower chassis fan.
    FAN1_VAL="`grep ${FAN1_TEXT} /tmp/sensors.out | awk '{print $3}'`"
    FAN2_VAL="`grep ${FAN2_TEXT} /tmp/sensors.out | awk '{print $3}'`"
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $4}'`"
    FANST0="$FAN1_VAL $FAN2_VAL $FAN3_VAL"
    FAN1_LOG=0
    FAN2_LOG=0
    FAN3_LOG=0
    ;;
    2466)
#  5 fans on 2U Tyan 2466, 2 cpu fans, 3 chassis fans
    FAN1_VAL="`grep ${FAN1_TEXT} /tmp/sensors.out | awk '{print $3}'`"
    FAN2_VAL="`grep ${FAN2_TEXT} /tmp/sensors.out | awk '{print $3}'`"
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $4}'`"
    FAN4_VAL="`grep ${FAN4_TEXT} /tmp/sensors.out | awk '{print $4}'`"
    FAN5_VAL="`grep ${FAN5_TEXT} /tmp/sensors.out | awk '{print $4}'`"
    FANST0="$FAN1_VAL $FAN2_VAL $FAN3_VAL $FAN4_VAL $FAN5_VAL"
    FAN1_LOG=0
    FAN2_LOG=0
    FAN3_LOG=0
    FAN4_LOG=0
    FAN5_LOG=0
#   Check for a jammed condition--key number is cpu temp of 255
#   for 2466 architecture
    if [ $FAN1_VAL -eq 0 ] || [ $FAN2_VAL -eq 0 ] || [ $FAN3_VAL -eq 0 ] || [ $FAN4_VAL -eq 0 ] || [ $FAN5_VAL -eq 0 ]
    then
	CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/+//' |  sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
	CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/+//' |  sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
#   255 is error code for unavailable/jammed sensors on 2466/2468
	if [ $CPU1_VAL -eq 255 ] && [ $CPU2_VAL -eq 255 ] 
	then
#
#  Set logical values to -1 (unavailable)..will detect sensors jammed below
#
	    FAN1_LOG=-1
	    FAN2_LOG=-1
	    FAN3_LOG=-1
	    FAN4_LOG=-1
	    FAN5_LOG=-1
	    JAMMED="YES"
	fi
    fi
    ;;
    X5DPA)
#In an X5DPA, only two fans, both chassis blowers
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN4_VAL="`grep ${FAN4_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FANST0="$FAN3_VAL $FAN4_VAL"
    FAN3_LOG=0
    FAN4_LOG=0
    if [ $FAN3_VAL -eq 0 ] || [ $FAN4_VAL -eq 0 ]
    then
	date >> /var/log/x5dpa_jamfan
	cat /tmp/sensors.out >> /var/log/x5dpa_jamfan
    fi    
#   In a case where the cpu temperature is -1, that is a key 
#   for jammed sensors/info unavailable.
    CPUTEMP="`grep CPU /tmp/sensors.out | cut -d'' -f1 |  sed -e 's/CPU:         //'`"
    CPU1_VAL="`echo $CPUTEMP | cut -d' ' -f1`"
    CPU2_VAL="`echo $CPUTEMP | cut -d' ' -f2`"
    if [ $CPU1_VAL -eq -1 ] && [ $CPU2_VAL -eq -1 ]
    then
	FAN3_LOG=-1
	FAN4_LOG=-1
	date >> /var/log/x5dpa_jamfan
	cat /tmp/sensors.out >> /var/log/x5dpa_jamfan
	JAMMED="YES"
    fi
    ;;
    X5DPA-TGM)
#In an X5DPA, only two fans, both chassis blowers
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN4_VAL="`grep ${FAN4_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FANST0="$FAN3_VAL $FAN4_VAL"
    FAN3_LOG=0
    FAN4_LOG=0
#   In a case where the cpu temperature is -1, that is a key 
#   for jammed sensors/info unavailable.
    CPU1_TMP="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'.' -f1`"
    CPU1_LEN=${#CPU1_TMP}
    CPU1_VAL=${CPU1_TMP:$CPU1_LEN-4:4} 
    CPU2_TMP="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'.' -f1`"
    CPU2_LEN=${#CPU2_TMP}
    CPU2_VAL=${CPU2_TMP:$CPU2_LEN-4:4} 
    if [ $CPU1_VAL -eq -1 ] && [ $CPU2_VAL -eq -1 ]
    then
	FAN3_LOG=-1
	FAN4_LOG=-1
	date >> /var/log/x5dpa_jamfan
	cat /tmp/sensors.out >> /var/log/x5dpa_jamfan
	JAMMED="YES"
    fi
    ;;
    X6DVL-EG)
#   Only 2 fans in X6DVL-EG, both chassis blowers
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN4_VAL="`grep ${FAN4_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FANST0="$FAN3_VAL $FAN4_VAL"
    FAN3_LOG=0
    FAN4_LOG=0
#   In a case where the cpu temperature is -1, that is a key
#   for jammed sensors/info unavailable.
    CPU1_TMP="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'.' -f1`"
    CPU1_LEN=${#CPU1_TMP}
    CPU1_VAL=${CPU1_TMP:$CPU1_LEN-4:4}
    CPU2_TMP="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'.' -f1`"
    CPU2_LEN=${#CPU2_TMP}
    CPU2_VAL=${CPU2_TMP:$CPU2_LEN-4:4}
    if [ $CPU1_VAL -eq -1 ] && [ $CPU2_VAL -eq -1 ]
    then
        FAN3_LOG=-1
        FAN4_LOG=-1
        date >> /var/log/x5dpa_jamfan
        cat /tmp/sensors.out >> /var/log/x5dpa_jamfan
        JAMMED="YES"
    fi
    ;;

    2723)
#In a 2723 (fnpc17-22) no readable fans but we have to put in
# the empty case so we don't have a error condition detected,
# e.g. the script should return -2 for all fan?_log variables, 
# not -1 
    FAN_LOG=-2
    FAN_LOGSEL=-2
    FAN_STRING=""
    FAN_SEL=""
    ;;    
    L440GX)
    FAN_LOG=-2
    ;;
    H8DAR)
#   Only 2 fans in H8DAR, both chassis blowers
#    Check a third value because it tends to be non-zero if 
#    things are not working right
    FAN3_VAL="`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN4_VAL="`grep ${FAN4_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN5_VAL="`grep ${FAN5_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN6_VAL="`grep ${FAN6_TEXT} /tmp/sensors.out | awk '{print $2}'`"
    FAN3_LOG=0
    FAN4_LOG=0
#   Check the CPU temps.. if they are bad, fans are likely to be bad too.
    CPU1_VAL=`egrep ${CPU1_TEXT}  /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
    CPU2_VAL=`egrep ${CPU2_TEXT}  /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
#   CPU0 val is the first of three sensors in raw sensors output, representing
#   a motherboard temp.
    CPU0_VAL=`egrep ${CPU0_TEXT}  /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
#
#   FAN5_VAL and FAN6_VAL should never be non-zero.  In case of 
#   jammed sensors, sometimes they are.
#
    if [ $FAN5_VAL -ne 0 ] || [ $FAN6_VAL -ne 0 ]
    then
       FAN3_LOG=-1
       FAN4_LOG=-1
       JAMMED="YES"
    fi
#
#   If both CPU temps exactly equal to the motherboard temp 
#   it is an error condition
#
    if [ $CPU2_VAL -eq $CPU0_VAL ] && [ $CPU1_VAL -eq $CPU0_VAL ] 
    then
        FAN3_LOG=-1
        FAN4_LOG=-1
        JAMMED="YES"
    fi
# 
#  Any CPU temp negative is an error condition
#
    if [ $CPU1_VAL -le 0 ] || [ $CPU2_VAL -le 0 ] 
    then
        FAN3_LOG=-1
        FAN4_LOG=-1
        JAMMED="YES"
    fi
#
#   temperature of 127C is an error condition
#
    if [ $CPU1_VAL -eq 127 ] || [ $CPU2_VAL -eq 127 ] 
    then
        FAN3_LOG=-1
        FAN4_LOG=-1
        JAMMED="YES"
    fi
#
#   temperature of 119C is an error condition
#
    if [ $CPU1_VAL -eq 119 ] || [ $CPU2_VAL -eq 119 ]
    then
        FAN3_LOG=-1
        FAN4_LOG=-1
        JAMMED="YES"
    fi
#
#   Both CPU temps equal to 80 is an error condition
#
    if [ $CPU1_VAL -eq 80 ] && [ $CPU2_VAL -eq 80 ] 
    then
	FAN3_LOG=-1
        FAN4_LOG=-1
        JAMMED="YES"
    fi
    FANST0="$FAN3_VAL $FAN4_VAL"
    ;;
    DELL)
    CPUTEMP="`grep Reading /tmp/sensors.out | sed -e 's/Reading                   : //'  -e 's/ RPM//' | cut -d '.' -f1`"
    FAN1_VAL="`echo $CPUTEMP | cut -d' ' -f$FAN1_LOC`"
    FAN2_VAL="`echo $CPUTEMP | cut -d' ' -f$FAN2_LOC`"
    FAN3_VAL="`echo $CPUTEMP | cut -d' ' -f$FAN3_LOC`"
    FAN4_VAL="`echo $CPUTEMP | cut -d' ' -f$FAN4_LOC`"
    FAN5_VAL="`echo $CPUTEMP | cut -d' ' -f$FAN5_LOC`"
    FAN6_VAL="`echo $CPUTEMP | cut -d' ' -f$FAN6_LOC`"
    if [ $FAN1_VAL -eq -1 ] 
    then
	FAN1_LOG=-1
	JAMMED="YES"
    fi
    if [ $FAN2_VAL -eq -1 ]
    then
	FAN2_LOG=-1
	JAMMED="YES"
    fi
    if [ $FAN3_VAL -eq -1 ]
    then
	FAN3_LOG=-1
	JAMMED="YES"
    fi
    if [ $FAN4_VAL -eq -1 ]
    then
	FAN4_LOG=-1
	JAMMED="YES"
    fi
    if [ $FAN5_VAL -eq -1 ]
    then
	FAN5_LOG=-1
	JAMMED="YES"
    fi
    if [ $FAN6_VAL -eq -1 ]
    then
	FAN6_LOG=-1
	JAMMED="YES"
    fi
    FANST0="$FAN1_VAL $FAN2_VAL $FAN3_VAL $FAN4_VAL $FAN5_VAL $FAN6_VAL"
    ;;

    *)
    FAN1_VAL=0
    FAN2_VAL=0
    FAN3_VAL=0
    FAN4_VAL=0
    FAN5_VAL=0
    FAN6_VAL=0
    FAN_LOG=-1
    FAN1_LOG=-1
    FAN2_LOG=-1
    FAN3_LOG=-1
    FAN4_LOG=-1
    FAN5_LOG=-1
    FAN6_LOG=-1
    ;;
    esac
fi
#  Handle the case of detecting jammed sensors correctly.
#  3 situations:
#  First detect of a possible jam, return cpu_log -1
#  and touch the sensors.jammed file
#  second jam situation, return cpu_log 1
#  "unavailable" but not jammed condition, but there is 
# a sensors_jammed file, return cpu_log 1
#
if [ $FAN1_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	FAN1_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "FAN1, sensors jammed, $FAN1_VAL"
	else
	    logger "FAN1, previous sensors jammed, now data unavailable, $FAN1_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "FAN1 First bad data, sensors possibly jammed, $FAN1_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "FAN1 First bad data, sensors possibly jammed, $FAN1_VAL" 
	    fi
	fi
    fi
fi
if [ $FAN2_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	FAN2_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "FAN2, sensors jammed, $FAN2_VAL"
	else
	    logger "FAN2, previous sensors jammed, now data unavailable, $FAN2_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "FAN2 First bad data, sensors possibly jammed, $FAN2_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "FAN2 First bad data, sensors possibly jammed, $FAN2_VAL" 
	    fi
	fi
    fi
fi
if [ $FAN3_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	FAN3_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "FAN3, sensors jammed, $FAN3_VAL"
	else
	    logger "FAN3, previous sensors jammed, now data unavailable, $FAN3_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "FAN3 First bad data, sensors possibly jammed, $FAN3_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "FAN3 First bad data, sensors possibly jammed, $FAN3_VAL" 
	    fi
	fi
    fi
fi
if [ $FAN4_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	FAN4_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "FAN4, sensors jammed, $FAN4_VAL"
	else
	    logger "FAN4, previous sensors jammed, now data unavailable, $FAN4_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "FAN4 First bad data, sensors possibly jammed, $FAN4_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "FAN4 First bad data, sensors possibly jammed, $FAN4_VAL" 
	    fi
	fi
    fi
fi
if [ $FAN5_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	FAN5_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "FAN5, sensors jammed, $FAN5_VAL"
	else
	    logger "FAN5, previous sensors jammed, now data unavailable, $FAN5_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "FAN5 First bad data, sensors possibly jammed, $FAN5_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "FAN5 First bad data, sensors possibly jammed, $FAN5_VAL" 
	    fi
	fi
    fi
fi
if [ $FAN6_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	FAN6_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "FAN6, sensors jammed, $FAN6_VAL"
	else
	    logger "FAN6, previous sensors jammed, now data unavailable, $FAN6_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "FAN6 First bad data, sensors possibly jammed, $FAN6_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "FAN6 First bad data, sensors possibly jammed, $FAN6_VAL" 
	    fi
	fi
    fi
fi
if [ $FAN1_LOG -eq -1 ] || [ $FAN2_LOG -eq -1 ] || [ $FAN3_LOG -eq -1 ] || [ $FAN4_LOG -eq -1 ] || [ $FAN5_LOG -eq -1 ] || [ $FAN6_LOG -eq -1 ]
then
    FAN_LOG=-1
fi
if [ $FAN1_LOG -eq 1 ] || [ $FAN2_LOG -eq 1 ] || [ $FAN3_LOG -eq 1 ] || [ $FAN4_LOG -eq 1 ] || [ $FAN5_LOG -eq 1 ] || [ $FAN6_LOG -eq 1 ]
then
    FAN_LOG=1
fi


    FANSTA="$FAN1_VAL $FAN2_VAL $FAN3_VAL $FAN4_VAL $FAN5_VAL $FAN6_VAL"
    if [ "$ALLFAN" == "YES" ]
    then
	FANSTRING="$FANSTA"
    else
	FANSTRING="$FANST0"
    fi
if [ $FAN1_LOG -ge 0 ] && [ $FAN1_LOG -ne 1 ]
then
    if [ $FAN1_VAL -le $FAN1_MIN ] && [ $FAN1_VAL -ge 0 ]
    then
	FAN1_LOG=2
    if [ "$FAN1_SLOW" == "YES" ]
	then
	    FAN1_LOG=4
	fi
    fi
    if [ $FAN1_VAL -eq 0 ] && [ $FAN1_LOG -ge 0 ]
    then
	FAN1_LOG=3
	if [ "$FAN1_OFF" == "YES" ]
	then
	    FAN1_LOG=5
	fi
    fi
fi
if [ $FAN2_LOG -ge 0 ] && [ $FAN2_LOG -ne 1 ]
then
    if [ $FAN2_VAL -le $FAN2_MIN ] && [ $FAN2_VAL -ge 0 ] 
    then
	FAN2_LOG=2
	if [ "$FAN2_SLOW" == "YES" ]
	then
	    FAN2_LOG=4
	fi
    fi
    if [ $FAN2_VAL -eq 0 ] && [ $FAN2_LOG -ge 0 ]
    then
	FAN2_LOG=3
	if [ "$FAN2_OFF" == "YES" ]
	then
	    FAN2_LOG=5
	fi
    fi
fi
if [ $FAN3_LOG -ge 0 ] && [ $FAN3_LOG -ne 1 ]
then
    if [ $FAN3_VAL -le $FAN3_MIN ] && [ $FAN3_VAL -ge 0 ] 
    then
	FAN3_LOG=2
	if [ "$FAN3_SLOW" == "YES" ]
	then
	    FAN3_LOG=4
	fi
    fi
    if [ $FAN3_VAL -eq 0 ] && [ $FAN3_LOG -ge 0 ]
    then
	FAN3_LOG=3
	if [ "$FAN3_OFF" == "YES" ]
	then
	    FAN3_LOG=5
	fi
    fi
fi
if [ $FAN4_LOG -ge 0 ] && [ $FAN4_LOG -ne 1 ]
then
    if [ $FAN4_VAL -le $FAN4_MIN ] && [ $FAN4_VAL -ge 0 ] 
    then
	FAN4_LOG=2
	if [ "$FAN4_SLOW" == "YES" ]
	then
	    FAN4_LOG=4
	fi
    fi
    if [ $FAN4_VAL -eq 0 ] && [ $FAN4_LOG -ge 0 ]
    then
	FAN4_LOG=3
	if [ "$FAN4_OFF" == "YES" ]
	then
	    FAN4_LOG=5
	fi
    fi
fi
if [ $FAN5_LOG -ge 0 ] && [ $FAN5_LOG -ne 1 ]
then
    if [ $FAN5_VAL -le $FAN5_MIN ] && [ $FAN5_VAL -ge 0 ] 
    then
	FAN5_LOG=2
	if [ "$FAN5_SLOW" == "YES" ]
	then
	    FAN5_LOG=4
	fi
    fi
    if [ $FAN5_VAL -eq 0 ] && [ $FAN5_LOG -ge 0 ]
    then
	FAN5_LOG=3
	if [ "$FAN5_OFF" == "YES" ]
	then
	    FAN5_LOG=5
	fi
    fi
fi
if [ $FAN6_LOG -ge 0 ] && [ $FAN6_LOG -ne 1 ]
then
    if [ $FAN6_VAL -le $FAN6_MIN ] && [ $FAN6_VAL -ge 0 ] 
    then
	FAN6_LOG=2
	if [ "$FAN6_SLOW" == "YES" ]
	then
	    FAN6_LOG=4
	fi
    fi
    if [ $FAN6_VAL -eq 0 ] && [ $FAN6_LOG -ge 0 ]
    then
	FAN6_LOG=3
	if [ "$FAN6_OFF" == "YES" ]
	then
	    FAN6_LOG=5
	fi
    fi
fi
    case $HWTYPE in
	370DLE)
	    if [ $FAN1_LOG -eq 2 ] || [ $FAN2_LOG -eq 2 ]
	    then
		FAN_LOG=2
            fi
	    if [ $FAN1_LOG -eq 3 ] || [ $FAN2_LOG -eq 3 ]
	    then
		FAN_LOG=3
            fi
	    if [ $FAN1_LOG -eq 4 ] || [ $FAN2_LOG -eq 4 ]
	    then
		FAN_LOG=4
            fi
	    if [ $FAN1_LOG -eq 5 ] || [ $FAN2_LOG -eq 5 ]
	    then
		FAN_LOG=5
            fi
        ;;
	2518)
	    if [ $NFAN -gt 1 ] 
	    then 
		if [ $FAN1_LOG -eq 2 ] || [ $FAN2_LOG -eq 2 ] || [ $FAN3_LOG -eq 2 ]
		then
		    FAN_LOG=2
		fi
		if [ $FAN1_LOG -eq 3 ] || [ $FAN2_LOG -eq 3 ] || [ $FAN3_LOG -eq 3 ]
		then
		    FAN_LOG=3
		fi
		if [ $FAN1_LOG -eq 4 ] || [ $FAN2_LOG -eq 4 ] || [ $FAN3_LOG -eq 4 ]
		then
		    FAN_LOG=4
		fi
		if [ $FAN1_LOG -eq 5 ] || [ $FAN2_LOG -eq 5 ] || [ $FAN3_LOG -eq 5 ]
		then
		    FAN_LOG=5
		fi
	    else
		FAN_LOG=$FAN3_LOG
	    fi
	    ;;
	2468) 
		if [ $FAN1_LOG -eq 2 ] || [ $FAN2_LOG -eq 2 ] || [ $FAN3_LOG -eq 2 ]
		then
		    FAN_LOG=2
		fi
		if [ $FAN1_LOG -eq 3 ] || [ $FAN2_LOG -eq 3 ] || [ $FAN3_LOG -eq 3 ]
		then
		    FAN_LOG=3
		fi
		if [ $FAN1_LOG -eq 4 ] || [ $FAN2_LOG -eq 4 ] || [ $FAN3_LOG -eq 4 ]
		then
		    FAN_LOG=4
		fi
		if [ $FAN1_LOG -eq 5 ] || [ $FAN2_LOG -eq 5 ] || [ $FAN3_LOG -eq 5 ]
		then
		    FAN_LOG=5
		fi
    		;;
	2466)
		if [ $FAN1_LOG -eq 2 ] || [ $FAN2_LOG -eq 2 ] || [ $FAN3_LOG -eq 2 ] || [ $FAN4_LOG -eq 2 ] || [ $FAN5_LOG -eq 2 ]
		then
		    FAN_LOG=2
		fi
		if [ $FAN1_LOG -eq 3 ] || [ $FAN2_LOG -eq 3 ] || [ $FAN3_LOG -eq 3 ] || [ $FAN4_LOG -eq 3 ] || [ $FAN5_LOG -eq 3 ]
		then
		    FAN_LOG=3
		fi
		if [ $FAN1_LOG -eq 4 ] || [ $FAN2_LOG -eq 4 ] || [ $FAN3_LOG -eq 4 ] || [ $FAN4_LOG -eq 4 ] || [ $FAN5_LOG -eq 4 ]
		then
		    FAN_LOG=4
		fi
		if [ $FAN1_LOG -eq 5 ] || [ $FAN2_LOG -eq 5 ] || [ $FAN3_LOG -eq 5 ] || [ $FAN4_LOG -eq 5 ] || [ $FAN5_LOG -eq 5 ]
		then
		    FAN_LOG=5
		fi
		;;
         X5DPA | X5DPA-TGM | X6DVL-EG | H8DAR)
	    if [ $FAN3_LOG -eq 2 ] || [ $FAN4_LOG -eq 2 ]
	    then
		FAN_LOG=2
            fi
	    if [ $FAN3_LOG -eq 3 ] || [ $FAN4_LOG -eq 3 ]
	    then
		FAN_LOG=3
            fi
	    if [ $FAN3_LOG -eq 4 ] || [ $FAN4_LOG -eq 4 ]
	    then
		FAN_LOG=4
            fi
	    if [ $FAN3_LOG -eq 5 ] || [ $FAN4_LOG -eq 5 ]
	    then
		FAN_LOG=5
            fi
	    ;;
	DELL)
		if [ $FAN1_LOG -eq 2 ] || [ $FAN2_LOG -eq 2 ] || [ $FAN3_LOG -eq 2 ] || [ $FAN4_LOG -eq 2 ] || [ $FAN5_LOG -eq 2 ] || [ $FAN6_LOG -eq 2 ] 
		then
		    FAN_LOG=2
		fi
		if [ $FAN1_LOG -eq 3 ] || [ $FAN2_LOG -eq 3 ] || [ $FAN3_LOG -eq 3 ] || [ $FAN4_LOG -eq 3 ] || [ $FAN5_LOG -eq 3 ] || [ $FAN6_LOG -eq 3 ] 
		then
		    FAN_LOG=3
		fi
		if [ $FAN1_LOG -eq 4 ] || [ $FAN2_LOG -eq 4 ] || [ $FAN3_LOG -eq 4 ] || [ $FAN4_LOG -eq 4 ] || [ $FAN5_LOG -eq 4 ] || [ $FAN6_LOG -eq 4 ]
		then
		    FAN_LOG=4
		fi
		if [ $FAN1_LOG -eq 5 ] || [ $FAN2_LOG -eq 5 ] || [ $FAN3_LOG -eq 5 ] || [ $FAN4_LOG -eq 5 ] || [ $FAN5_LOG -eq 5 ] || [ $FAN6_LOG -eq 5 ] 
		then
		    FAN_LOG=5
		fi
		;;
	 2723)
            ;;
         L440GX)
	    ;;
	    *) 
	    FAN_LOG=-1
	    ;;
    esac
    case $NF in
	0) 
	FAN_SEL=$FANSTRING
	FAN_LOGSEL=$FAN_LOG
        ;;
	1)
	FAN_SEL=$FAN1_VAL 
	FAN_LOGSEL=$FAN1_LOG
        ;;
	2) 
	FAN_SEL=$FAN2_VAL
 	FAN_LOGSEL=$FAN2_LOG
        ;;
	3) 
	FAN_SEL=$FAN3_VAL
 	FAN_LOGSEL=$FAN3_LOG
        ;;
	4)
	FAN_SEL=$FAN4_VAL
 	FAN_LOGSEL=$FAN4_LOG
        ;;
	5) 
	FAN_SEL=$FAN5_VAL
 	FAN_LOGSEL=$FAN5_LOG
        ;;
	6) 
	FAN_SEL=$FAN6_VAL
 	FAN_LOGSEL=$FAN6_LOG
        ;;
    esac  
if [ "$FAN_LOGSEL" == "" ] 
then
    $FAN_LOGSEL=-1
    date >> /var/log/fanprop.log
    echo "had to set a null fan_logsel"
    cat /tmp/sensors.out >> /var/log/fanprob.log
fi
if [ $FAN_LOGSEL -gt 0 ]
then
    date >> /var/log/fanprob.log
    cat /tmp/sensors.out >> /var/log/fanprob.log
fi
if [ "$LOGRES" == "LOGICAL" ]
then
    echo $FAN_LOGSEL
else
    echo "$FAN_SEL"
fi
#
#Fix the record files for history
#
case $FAN1_LOG in 
    0) 
    if [ -s /var/log/fan1_slow ]
    then
	rm /var/log/fan1_slow
    fi
    if [ -s /var/log/fan1_off ]
    then
	rm /var/log/fan1_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    1)
    if [ -s /var/log/fan1_slow ]
    then
	rm /var/log/fan1_slow 
    fi
    if [ -s /var/log/fan1_off ]
    then
	rm /var/log/fan1_off
    fi
    ;;
    2) 
    if [ ! -s /var/log/fan1_slow ]
    then
	echo "CPUfan 1 slow, $FAN1_VAL, `date`" > /var/log/fan1_slow
	cat /tmp/sensors.out >> /var/log/fan1_slow
	logger "CPUFan1 slow, $FAN1_VAL"
    fi
    if [ -s /var/log/fan1_off ]
    then
	rm /var/log/fan1_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    3) 
    if [ ! -s /var/log/fan1_off ]
    then
	echo "CPUFan 1 off , $FAN1_VAL, `date`" > /var/log/fan1_off
	cat /tmp/sensors.out >> /var/log/fan1_off
	logger "CPUFan1 off, $FAN1_VAL"
    fi
    if [ -s /var/log/fan1_slow ]
    then
	rm /var/log/fan1_slow
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    4) 
    if [ -s /var/log/fan1_off ] 
    then
	rm /var/log/fan1_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "CPUFan1 continued slow, $FAN1_VAL"
    ;;
    5) 
    if [ -s /var/log/fan1_slow ] 
    then
	rm /var/log/fan1_slow 
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "CPUFan1 continued stopped, $FAN1_VAL"
    ;;
esac
case $FAN2_LOG in 
    0) 
    if [ -s /var/log/fan2_slow ]
    then
	rm /var/log/fan2_slow
    fi
    if [ -s /var/log/fan2_off ]
    then
	rm /var/log/fan2_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    1)
    if [ ! -s /var/log/sensors.jammed ] 
    then
	echo "sensors jammed `date` " > /var/log/sensors.jammed
	cat /tmp/sensors.out >> /var/log/sensors.jammed
	logger "sensors jammed `date`"
    fi
    if [ -s /var/log/fan2_slow ]
    then
	rm /var/log/fan2_slow 
    fi
    if [ -s /var/log/fan2_off ]
    then
	rm /var/log/fan2_off
    fi
    ;;
    2) 
    if [ ! -s /var/log/fan2_slow ]
    then
	echo "CPUfan2 slow, $FAN2_VAL, `date`" > /var/log/fan2_slow
	cat /tmp/sensors.out >> /var/log/fan2_slow
	logger "CPUFan2 slow, $FAN2_VAL"
    fi
    if [ -s /var/log/fan2_off ]
    then
	rm /var/log/fan2_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    3) 
    if [ ! -s /var/log/fan2_off ]
    then
	echo "CPUFan2 off , $FAN2_VAL, `date`" > /var/log/fan2_off
	cat /tmp/sensors.out >> /var/log/fan2_off
	logger "CPUFan2 off, $FAN2_VAL"
    fi
    if [ -s /var/log/fan2_slow ]
    then
	rm /var/log/fan2_slow
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    4) 
    if [ -s /var/log/fan2_off ] 
    then
	rm /var/log/fan2_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "CPUFan2 continued slow, $FAN2_VAL"
    ;;
    5) 
    if [ -s /var/log/fan2_slow ] 
    then
	rm /var/log/fan2_slow 
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "CPUFan2 continued stopped, $FAN2_VAL"
    ;;
esac
case $FAN3_LOG in 
    0) 
    if [ -s /var/log/fan3_slow ]
    then
	rm /var/log/fan3_slow
    fi
    if [ -s /var/log/fan3_off ]
    then
	rm /var/log/fan3_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    1)
    if [ ! -s /var/log/sensors.jammed ] 
    then
	echo "sensors jammed `date` " > /var/log/sensors.jammed
	cat /tmp/sensors.out >> /var/log/sensors.jammed
	logger "sensors jammed `date`"
    fi
    if [ -s /var/log/fan3_slow ]
    then
	rm /var/log/fan3_slow 
    fi
    if [ -s /var/log/fan3_off ]
    then
	rm /var/log/fan3_off
    fi
    ;;
    2) 
    if [ ! -s /var/log/fan3_slow ]
    then
	echo "Chassis Fan 1 slow, $FAN3_VAL, `date`" > /var/log/fan3_slow
	cat /tmp/sensors.out >> /var/log/fan3_slow
	logger "Chassis Fan 1 slow, $FAN3_VAL"
    fi
    if [ -s /var/log/fan3_off ]
    then
	rm /var/log/fan3_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    3) 
    if [ ! -s /var/log/fan3_off ]
    then
	echo "Chassis Fan 1 off , $FAN3_VAL, `date`" > /var/log/fan3_off
	cat /tmp/sensors.out >> /var/log/fan3_off
	logger "Chassis Fan 1 off, $FAN3_VAL"
    fi
    if [ -s /var/log/fan3_slow ]
    then
	rm /var/log/fan3_slow
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    4) 
    if [ -s /var/log/fan3_off ] 
    then
	rm /var/log/fan3_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 1 continued slow, $FAN3_VAL"
    ;;
    5) 
    if [ -s /var/log/fan3_slow ] 
    then
	rm /var/log/fan3_slow 
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 1 continued stopped, $FAN3_VAL"
    ;;
esac
case $FAN4_LOG in 
    0) 
    if [ -s /var/log/fan4_slow ]
    then
	rm /var/log/fan4_slow
    fi
    if [ -s /var/log/fan4_off ]
    then
	rm /var/log/fan4_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    1)
    if [ ! -s /var/log/sensors.jammed ] 
    then
	echo "sensors jammed `date` " > /var/log/sensors.jammed
	cat /tmp/sensors.out >> /var/log/sensors.jammed
	logger "sensors jammed `date`"
    fi
    if [ -s /var/log/fan4_slow ]
    then
	rm /var/log/fan4_slow 
    fi
    if [ -s /var/log/fan4_off ]
    then
	rm /var/log/fan4_off
    fi
    ;;
    2) 
    if [ ! -s /var/log/fan4_slow ]
    then
	echo "CPUFan 4 slow, $FAN4_VAL, `date`" > /var/log/fan4_slow
	cat /tmp/sensors.out >> /var/log/fan4_slow
	logger "Chassis Fan 2 slow, $FAN4_VAL"
    fi
    if [ -s /var/log/fan4_off ]
    then
	rm /var/log/fan4_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    3) 
    if [ ! -s /var/log/fan4_off ]
    then
	echo "Chassis Fan 2 off , $FAN4_VAL, `date`" > /var/log/fan4_off
	cat /tmp/sensors.out >> /var/log/fan4_off
	logger "Chassis Fan 2 off, $FAN4_VAL"
    fi
    if [ -s /var/log/fan4_slow ]
    then
	rm /var/log/fan4_slow
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    4) 
    if [ -s /var/log/fan4_off ] 
    then
	rm /var/log/fan4_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 2 continued slow, $FAN4_VAL"
    ;;
    5) 
    if [ -s /var/log/fan4_slow ] 
    then
	rm /var/log/fan4_slow 
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 2 continued stopped, $FAN4_VAL"
    ;;
esac
case $FAN5_LOG in 
    0) 
    if [ -s /var/log/fan5_slow ]
    then
	rm /var/log/fan5_slow
    fi
    if [ -s /var/log/fan5_off ]
    then
	rm /var/log/fan5_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    1)
    if [ ! -s /var/log/sensors.jammed ] 
    then
	echo "sensors jammed `date` " > /var/log/sensors.jammed
	cat /tmp/sensors.out >> /var/log/sensors.jammed
	logger "sensors jammed `date`"
    fi
    if [ -s /var/log/fan5_slow ]
    then
	rm /var/log/fan5_slow 
    fi
    if [ -s /var/log/fan5_off ]
    then
	rm /var/log/fan5_off
    fi
    ;;
    2) 
    if [ ! -s /var/log/fan5_slow ]
    then
	echo "CPUfan 1 slow, $FAN5_VAL, `date`" > /var/log/fan5_slow
	cat /tmp/sensors.out >> /var/log/fan5_slow
	logger "Chassis Fan 3 slow, $FAN5_VAL"
    fi
    if [ -s /var/log/fan5_off ]
    then
	rm /var/log/fan5_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    3) 
    if [ ! -s /var/log/fan5_off ]
    then
	echo "Chassis Fan 3 off , $FAN5_VAL, `date`" > /var/log/fan5_off
	cat /tmp/sensors.out >> /var/log/fan5_off
	logger "Chassis Fan 3 off, $FAN5_VAL"
    fi
    if [ -s /var/log/fan5_slow ]
    then
	rm /var/log/fan5_slow
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    4) 
    if [ -s /var/log/fan5_off ] 
    then
	rm /var/log/fan5_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 3 continued slow, $FAN5_VAL"
    ;;
    5) 
    if [ -s /var/log/fan5_slow ] 
    then
	rm /var/log/fan5_slow 
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 3 continued stopped, $FAN5_VAL"
    ;;
esac
case $FAN6_LOG in 
    0) 
    if [ -s /var/log/fan6_slow ]
    then
	rm /var/log/fan6_slow
    fi
    if [ -s /var/log/fan6_off ]
    then
	rm /var/log/fan6_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    1)
    if [ ! -s /var/log/sensors.jammed ] 
    then
	echo "sensors jammed `date` " > /var/log/sensors.jammed
	cat /tmp/sensors.out >> /var/log/sensors.jammed
	logger "sensors jammed `date`"
    fi
    if [ -s /var/log/fan6_slow ]
    then
	rm /var/log/fan6_slow 
    fi
    if [ -s /var/log/fan6_off ]
    then
	rm /var/log/fan6_off
    fi
    ;;
    2) 
    if [ ! -s /var/log/fan6_slow ]
    then
	echo "Chassis Fan 4 slow, $FAN6_VAL, `date`" > /var/log/fan6_slow
	cat /tmp/sensors.out >> /var/log/fan6_slow
	logger "Chassis Fan 4 slow, $FAN6_VAL"
    fi
    if [ -s /var/log/fan6_off ]
    then
	rm /var/log/fan6_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    3) 
    if [ ! -s /var/log/fan6_off ]
    then
	echo "CPUFan 1 off , $FAN6_VAL, `date`" > /var/log/fan6_off
	cat /tmp/sensors.out >> /var/log/fan6_off
	logger "Chassis Fan 4 off, $FAN6_VAL"
    fi
    if [ -s /var/log/fan6_slow ]
    then
	rm /var/log/fan6_slow
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    ;;
    4) 
    if [ -s /var/log/fan6_off ] 
    then
	rm /var/log/fan6_off
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 4 continued slow, $FAN6_VAL"
    ;;
    5) 
    if [ -s /var/log/fan6_slow ] 
    then
	rm /var/log/fan6_slow 
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "Chassis Fan 4 continued stopped, $FAN6_VAL"
    ;;
esac
#
#  Only run ganglia for case of NF=0--this will stop doing a large
#  number of broadcasts in rapid succession
#
#
if [ $NF -eq 0 ] 
then
#  Ganglia section
#  Now broadcast the values to ganglia using the "gmetric" tool
#  modified from a script by Mike Snitzer
#
GMETRIC_BIN=/usr/bin/gmetric
# Get the multicast channel to send out to:
#
#   In Fermi Farms config mcast_channel line is often commented out
#   use the default mcast_channel if so (or if /etc/gmond.conf doesn't exist)
MCASTCHAN=""
if [ -e /etc/gmond.conf ]
then
    MCASTCHAN="`egrep ^mcast_channel /etc/gmond.conf | awk '{print $2}'`"
    if [ "$MCASTCHAN" == "" ]
    then
	GMETRIC="${GMETRIC_BIN} "
    else
	GMETRIC="${GMETRIC_BIN} -c ${MCASTCHAN} "
    fi
fi
	
#   establish a base commandline
#
#   check to be sure gmond is running
#
`/sbin/service gmond status > /dev/null`
if [ $? -eq 0 ] && [ -x $GMETRIC_BIN ] 
then
#
# send fan speeds
#
    $GMETRIC  --type int16 --name "fan_ok" --units "binary" --value $FAN_LOG
    if [ $FAN1_VAL -ne -2 ]
    then
	$GMETRIC  --type int16 --name "cpufan_1" --units "RPM" --value $FAN1_VAL
    fi
    if [ $FAN2_VAL -ne -2 ]
    then
	$GMETRIC  --type int16 --name "cpufan_2" --units "RPM" --value $FAN2_VAL
    fi
    if [ $FAN3_VAL -ne -2 ]
    then
	$GMETRIC  --type int16 --name "chassisfan_1" --units "RPM" --value $FAN3_VAL
    fi
    if [ $FAN3_VAL -ne -2 ]
    then
	$GMETRIC  --type int16 --name "chassisfan_2" --units "RPM" --value $FAN4_VAL
    fi
    if [ $FAN4_VAL -ne -2 ]
    then
	$GMETRIC  --type int16 --name "chassisfan_3" --units "RPM" --value $FAN5_VAL
    fi
    if [ $FAN6_VAL -ne -2 ]
    then
	$GMETRIC  --type int16 --name "chassisfan_4" --units "RPM" --value $FAN6_VAL
    fi
fi
fi
