#!/bin/sh
# Last modified 4/12/05, merging X6DVL-EG changes of v6 and intrans changes
# of v5 which somehow weren't included in v6., also fixing bug where
# degree sign doesn't print sometimes.
#This is supposed to be a generic wrapper for sensors
#on various nodes to run the sensors and return the two 
#cpu temperatures to ngop.
#We will store hardware type in /etc/hwtype
#That's an executable shell script that sets the levels
#of acceptable temperature and the strings to grep for to get them.
#
# With no arguments it will return the two CPU temperatures
# With argument -l (lower case l) it will return an integer severity
# -1 information not available
# 0 normal
# 1 sensors jammed (ngop blue alarm + event)
# 2 overtemp 5 degrees first time (ngop yellow alarm) 
#   (note our ngop agents are currently programmed to ignore this level).
# 3 overtemp 10 degrees first time (ngop red alarm)
# 4 continuous overtemp 5 degrees (ngop yellow alarm + event)
# 5 continuous overtemp 10 degrees (ngop red alarm + event)
# 6 over emergency temperature (ngop red alarm and event plus Email action)
# 7 continuous over emergency temperature (ngop shutdown action)
# With argument -n [1 | 2] it will return only the first or 
# only the second CPU temperature, modified by -l above as necessary
# Note it will always report all values to ganglia.
#
if [ -x /etc/hwtype ]
then
#Set values of FIRSTLEV and SECONDLEV for old /etc/hwtype that
#may not have them set
    FIRSTLEV=5
    SECONDLEV=10   
#set values of CPU_LOC and FAN_LOC for old /etc/hwtype that may not
#have them set (Only matters for Dell hardware type)
    CPU_LOC1=1
    CPU_LOC2=2
    FAN_LOC1=6
    FAN_LOC2=7
    FAN_LOC3=8
    FAN_LOC4=9
    FAN_LOC5=10
    FAN_LOC6=11
    . /etc/hwtype
else
    HWTYPE="2466"
    CPU1_IDLE=55
    CPU2_IDLE=55
    CPU1_LEVEL=65
    CPU2_LEVEL=65
    CPU1_EMERG=75
    CPU2_EMERG=75
    CPU1_TEXT="'Near CPU1'"
    CPU2_TEXT="'Near CPU2'"
    FIRSTLEV=5
    SECONDLEV=10
#set values of CPU_LOC and FAN_LOC for old /etc/hwtype that may not
#have them set (Only matters for Dell hardware type)
    CPU_LOC1=1
    CPU_LOC2=2
    FAN_LOC1=6
    FAN_LOC2=7
    FAN_LOC3=8
    FAN_LOC4=9
    FAN_LOC5=10
    FAN_LOC6=11
fi 
## Source /etc/profile.d/lang.sh to make sure language
## flag is set right, necessary to make the degree sign come out right.
if [ -r /etc/profile.d/lang.sh ]
then
    . /etc/profile.d/lang.sh
fi
# Look for the node-specific temperature file
if [ -x /etc/hwtype.node ]
then
    . /etc/hwtype.node
fi
#
# Check pre-existing state files
# Files have to be there and be at least 3 minutes old
# Otherwise they will be pre-existing right away since this
# wrapper script is executed several times by the ngop agent on 
# every cycle
#
WAS_TEMPALERT="NO"
if [ -s /var/log/tempcpu1_5 ]
then
    WAS_TEMPALERT="YES"
    T15="`find /var/log/tempcpu1_5 -mmin +3 -print`"
    if [ "$T15" != "" ] 
    then
	TEMPCPU1_5="YES"
    fi
fi
if [ -s /var/log/tempcpu2_5 ]
then
    WAS_TEMPALERT="YES"
    T25="`find /var/log/tempcpu2_5 -mmin +3 -print`"
    if [ "$T25" != "" ] 
    then
	TEMPCPU2_5="YES"
    fi
fi
if [ -s /var/log/tempcpu1_10 ]
then
    WAS_TEMPALERT="YES"
    T110="`find /var/log/tempcpu1_10 -mmin +3 -print`"
    if [ "$T110" != "" ] 
    then
	TEMPCPU1_10="YES"
    fi
fi
if [ -s /var/log/tempcpu2_10 ]
then
    WAS_TEMPALERT="YES"
    T210="`find /var/log/tempcpu2_10 -mmin +3 -print`"
    if [ "$T210" != "" ] 
    then
	TEMPCPU2_10="YES"
    fi
fi
if [ -s /var/log/tempcpu1_emerg ]
then
    WAS_TEMPALERT="YES"
    T1EMERG="`find /var/log/tempcpu1_emerg -mmin +3 -print`"
    if [ "$T1EMERG" != "" ] 
    then
	TEMPCPU1_EMERG="YES"
    fi
fi
if [ -s /var/log/tempcpu2_emerg ]
then
    WAS_TEMPALERT="YES"
    T2EMERG="`find /var/log/tempcpu2_emerg -mmin +3 -print`"
    if [ "$T2EMERG" != "" ] 
    then
	TEMPCPU2_EMERG="YES"
    fi
fi
if [ -s /var/log/sensors.jammed ]
then
    T2JAM="`find /var/log/sensors.jammed -mmin +3 -print`"
    if [ "$T2JAM" != "" ]
    then
	SENSORS_JAMMED="YES"
    fi
fi
#
# Check and see if we are monitoring a possible rise in temperature
# from idle-to-loaded machine transition--the file must 
# exist and be at least 10 minutes old.
WAS_INTRANS="NO"
INTRANS="NO"
EXIST_INTRANS="NO"
if [ -e /var/log/system.intrans ]
then
    EXIST_INTRANS="YES"
    SYSINTRANS="`find /var/log/system.intrans -mmin +10 -print`"
    if [ "$SYSINTRANS" != "" ] 
    then
	WAS_INTRANS="YES"
    fi
fi
# Check and see if the system has been idle within last 10 minutes
# We touch /var/log/system.idle every time it is.
#
WAS_IDLE="NO"
if [ -e /var/log/system.idle ]
then
    SYSIDLE="`find /var/log/system.idle -mmin +10 -print`"
    if [ "$SYSIDLE" == "" ] 
    then
	WAS_IDLE="YES"
    else
	rm /var/log/system.idle
    fi
fi
# Check and see if the system has been loaded within last 10 minutes
# We touch /var/log/system.loaded every time it is.
#
WAS_LOADED="NO"
if [ -e /var/log/system.loaded ]
then
    SYSLOADED="`find /var/log/system.loaded -mmin +10 -print`"
    if [ "$SYSLOADED" == "" ] 
    then
#   file exists but is not 10 minutes old--that means we had load
#   sometime in last 10 minutes
	WAS_LOADED="YES"
    else
#   file exists and is 10 minutes old, remove it.
#
	rm /var/log/system.loaded
    fi
fi
#
# Parse the arguments of the script
LOGRES="NUMERIC"
JAMMED="NO"
NCPU=0
CPU_LOG=0
CPU1_LOG=0
CPU2_LOG=0
CPU1_VAL=0
CPU2_VAL=0
NEWDATA="NO"
while [ $# -ne 0 ] 
do
    case $1 in 
	-l) LOGRES="LOGICAL" ; shift 1 ;;
        -n) test $# -lt 2 && { echo "invalid arguments to -n" ; exit 1 ; }
	    NCPU=$2 ; shift 2 ;;
        *)  echo "invalid arguments" ; exit 1 ;;
    esac
done
###
### Check for test files
###
TESTING="NO"
if [ -e /var/log/testcpu1_5 ]
then
    TESTING="YES"
    CPU1_LOG=4
    CPU_LOG=4
    CPU1_VAL=`expr $CPU1_LEVEL + $FIRSTLEV + 1`
fi
if [ -e /var/log/testcpu2_5 ]
then
    TESTING="YES"
    CPU2_LOG=4
    CPU_LOG=4
    CPU2_VAL=`expr $CPU2_LEVEL + $FIRSTLEV + 1`
fi
if [ -e /var/log/testcpu1_10 ]
then
    TESTING="YES"
    CPU1_LOG=5
    CPU_LOG=5
    CPU1_VAL=`expr $CPU1_LEVEL + $SECONDLEV + 1`
fi
if [ -e /var/log/testcpu2_10 ]
then
    TESTING="YES"
    CPU2_LOG=5
    CPU2_VAL=`expr $CPU2_LEVEL + $SECONDLEV + 1`
fi
if [ -e /var/log/testcpu1_emerg ]
then
    TESTING="YES"
    CPU1_LOG=7
    CPU_LOG=7
    CPU1_VAL=`expr $CPU1_EMERG + 1`
fi
if [ -e /var/log/testcpu2_emerg ] 
then
    TESTING="YES"
    CPU2_LOG=7
    CPU_LOG=7
    CPU2_VAL=`expr $CPU2_EMERG + 1`
fi
if [ "$TESTING" == "YES" ]
then 
    CPU1_TEST=$CPU1_LEVEL
    CPU2_TEST=$CPU2_LEVEL
fi
#Remove any sensors.out files older than 3 minutes old
#leave them there if they are younger than 3 minutes.
#This allows us to run sensors just once for multiple
#calls of sensors_wrap.sh and fan_wrap.sh by the ngop agent
if [ -s /tmp/sensors.out ]
then
    find /tmp/sensors.out -mmin +3 -exec rm -f '{}' ';'
fi
#
# Get the load (1 min load avg)
LOAD=`uptime | awk '{print $(NF-2)}' | sed -e 's/,//' -e 's/\.//'`
LOADNUM=`uptime | awk '{print $(NF-2) $(NF-1) $(NF)}'`
# Run sensors again only if we need to.
if [ ! -s /tmp/sensors.out ] 
then
#
#Check to see if sensors is already running (if so it's an error condition
# and the process is probably stuck
    NSENSORS=`ps -ef | grep /usr/bin/sensors | grep -v grep | wc -l`
    NSYSSTAT=`ps -ef | grep /root/bin/sysstat/sysstat | grep -v grep | wc -l`
    NOMREPORT=`ps -ef | grep /usr/bin/omreport | grep -v grep | wc -l`
    if [ $NSENSORS -eq 0 ] && [ $NSYSSTAT -eq 0 ] && [ $NOMREPORT -eq 0 ]
    then
#
	if [ "${HWTYPE}" == "L440GX" ] && [ -x /root/bin/sysstat/sysstat ]
	then
	    /root/bin/sysstat/sysstat > /tmp/sensors.out 2>/tmp/sensors.err
            NEWDATA="YES"
	else
	    if [ "${HWTYPE}" == "DELL" ] && [ -x /usr/bin/omreport ]
            then
                /usr/bin/omreport chassis temps > /tmp/sensors.out 2>/tmp/sensors.err ; /usr/bin/omreport chassis fans >> /tmp/sensors.out 2>>/tmp/sensors.err
                 NEWDATA="YES"
            else
	         if [ -x /usr/bin/sensors ]
	         then
		    /usr/bin/sensors > /tmp/sensors.out 2>/tmp/sensors.err
		    NEWDATA="YES"
	         fi
            fi
	fi
    else
	CPU_LOG=-1
	CPU1_LOG=-1
	CPU2_LOG=-1
	CPU1_VAL=0
	CPU2_VAL=0
    fi
fi
#should grep for error messages in /tmp/sensors.out here
if [ "$TESTING" == "NO" ]
then
if [ -s /tmp/sensors.out ] 
then
    case "$HWTYPE" in
    L440GX)
    CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d' ' -f5 | cut -d'.' -f1`"
    CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d' ' -f5 | cut -d'.' -f1`"
    ;;
    370DLE)
    CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'+' -f2 | cut -d'' -f1`"
    CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'+' -f2 | cut -d'' -f1 `"
    ;;
    2518)
    CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
    CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
    ;;
    2466|2468)
    CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/+//' |  sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
    CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/+//' |  sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
#   255 is error code for unavailable/jammed sensors on 2466/2468
    if [ $CPU1_VAL -eq 255 ] 
    then
	CPU1_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq 255 ] 
    then
	CPU2_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU1_VAL -eq 0 ] 
    then
	CPU1_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq 0 ] 
    then
	CPU2_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
	CPU_LOG=-1
    fi
    ;;
    X6DVL-EG)
#    CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/+//' |  sed -e 's/C  (high =  //' | cut -d'.' -f1`"
#    CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'+' -f2 | sed -e 's/+//' |  sed -e 's/C  (high =  //' | cut -d'.' -f1`"
#   255 is error code for unavailable/jammed sensors on X6DVL-EG
     CPU1_VAL=`egrep ${CPU1_TEXT} /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
     CPU2_VAL=`egrep ${CPU2_TEXT} /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
    if [ $CPU1_VAL -le 0 ] 
    then
        CPU1_LOG=-1
    fi
    if [ $CPU2_VAL -le 0 ]
    then
        CPU2_LOG=-1
    fi
    if [ $CPU1_VAL -eq 255 ]
    then
        CPU1_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq 255 ]
    then
        CPU2_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
    CPU_LOG=-1
    fi
    ;;
    X5DPA)
    CPUTEMP="`grep CPU /tmp/sensors.out | cut -d'' -f1 |  sed -e 's/CPU:         //'`"
    CPU1_VAL="`echo $CPUTEMP | cut -d' ' -f1`"
    CPU2_VAL="`echo $CPUTEMP | cut -d' ' -f2`"
    if [ $CPU1_VAL -eq -1 ] 
    then
	CPU1_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq -1 ]
    then
	CPU2_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
	CPU_LOG=-1
    fi
    ;;
#Case X5DPA-TGM is deprecated now, we run all our X5DPA-TGM 
#boards as X5DPA
    X5DPA-TGM)
    CPU1_TMP="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'.' -f1`"
    CPU1_LEN=${#CPU1_TMP}
    CPU1_VAL=${CPU1_TMP:$CPU1_LEN-4:4} 
    CPU2_TMP="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'.' -f1`"
    CPU2_LEN=${#CPU2_TMP}
    CPU2_VAL=${CPU2_TMP:$CPU2_LEN-4:4} 
    if [ $CPU1_VAL -eq -1 ] 
    then
	CPU1_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq -1 ]
    then
	CPU2_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
	CPU_LOG=-1
    fi
    ;;
    2723)
    CPU1_VAL="`grep ${CPU1_TEXT} /tmp/sensors.out | cut -d'+' -f2 |  sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
    CPU2_VAL="`grep ${CPU2_TEXT} /tmp/sensors.out | cut -d'+' -f2 |  sed -e 's/C  (limit =  //' | cut -d'.' -f1`"
#   255 is error code for unavailable/jammed sensors on 2723
    if [ $CPU1_VAL -eq 255 ] 
    then
	CPU1_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq 255 ] 
    then
	CPU2_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
	CPU_LOG=-1
    fi
    ;;
   H8DAR)
    CPU1_VAL=`egrep ${CPU1_TEXT}  /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
    CPU2_VAL=`egrep ${CPU2_TEXT}  /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
#   CPU0 val is the first of three sensors in raw sensors output, representing
#   a motherboard temp.
    CPU0_VAL=`egrep ${CPU0_TEXT}  /tmp/sensors.out | awk '{print $2}' | cut -d '.' -f1`
  
    FAN3_VAL=`grep ${FAN3_TEXT} /tmp/sensors.out | awk '{print $2}'`
    FAN4_VAL=`grep ${FAN4_TEXT} /tmp/sensors.out | awk '{print $2}'`  
#   These 2 fans represent sensors that aren't hooked up, should be 
#   zero in the case of a working sensor.
    FAN5_VAL=`grep ${FAN5_TEXT} /tmp/sensors.out | awk '{print $2}'`  
    FAN6_VAL=`grep ${FAN6_TEXT} /tmp/sensors.out | awk '{print $2}'`  
#
#   If both CPU temps exactly equal to the motherboard temp 
#   it is an error condition
#
    if [ $CPU2_VAL -eq $CPU0_VAL ] && [ $CPU1_VAL -eq $CPU0_VAL ] 
    then
        CPU1_LOG=-1
        CPU2_LOG=-1
        JAMMED="YES"
    fi
# 
#  Any CPU temp negative is an error condition
#
    if [ $CPU1_VAL -le 0 ]
    then
        CPU1_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU2_VAL -le 0 ]
    then
        CPU2_LOG=-1
        JAMMED="YES"
    fi
#
#   temperature of 127C is an error condition
#
    if [ $CPU1_VAL -eq 127 ]
    then
        CPU1_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq 127 ]
    then
        CPU2_LOG=-1
        JAMMED="YES"
    fi
#
#   temperature of 119C is an error condition
#
    if [ $CPU1_VAL -eq 119 ]
    then
        CPU1_LOG=-1
        JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq 119 ]
    then
        CPU2_LOG=-1
        JAMMED="YES"
    fi
#
#   Both CPU temps equal to 80 is an error condition
#
    if [ $CPU1_VAL -eq 80 ] && [ $CPU2_VAL -eq 80 ] 
    then
	CPU1_LOG=-1
        CPU2_LOG=-1
        JAMMED="YES"
    fi
#
#   On this architecture FAN5_VAL and FAN6_VAL should never be non-zero
#   but when the sensors are jammed sometimes they are,
    if [ $FAN5_VAL -ne 0 ] || [ $FAN6_VAL -ne 0 ] 
    then
        CPU1_LOG=-1
        CPU2_LOG=-1
        JAMMED="YES"
    fi

    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
        CPU_LOG=-1
    fi
    ;;
    DELL)
    CPUTEMP="`grep Reading /tmp/sensors.out | sed -e 's/Reading                   : //'  -e 's/ RPM//' | cut -d '.' -f1`"
    CPU1_VAL="`echo $CPUTEMP | cut -d' ' -f$CPU_LOC1`"
    CPU2_VAL="`echo $CPUTEMP | cut -d' ' -f$CPU_LOC2`"
    if [ $CPU1_VAL -eq -1 ] 
    then
	CPU1_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU2_VAL -eq -1 ]
    then
	CPU2_LOG=-1
	JAMMED="YES"
    fi
    if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ]
    then
	CPU_LOG=-1
    fi
    ;;
    *)
    CPU1_VAL=0
    CPU2_VAL=0
    CPU_LOG=-1
    CPU1_LOG=-1
    CPU2_LOG=-1
    ;;
    esac
#
#   Determine the testing level based on load factor
#
    IS_IDLE="NO"
# Now load actually is load * 100, we use loaded threshold if 
# more than 10% loaded.
    if [ $LOAD -lt 10 ] 
    then
	touch /var/log/system.idle
	CPU1_TEST=$CPU1_IDLE
	CPU2_TEST=$CPU2_IDLE
        IS_IDLE="YES"
#      The trailing transition--loaded-to-idle
#      If system is idle now but was loaded within last 10 minutes
#      we take whichever of the two thresholds, idle or loaded, is greater
#
	if [ "$WAS_LOADED" == "YES" ] 
	then
	    if [ $CPU1_LEVEL -gt $CPU1_IDLE ]
	    then
		CPU1_TEST=$CPU1_LEVEL
            fi
	    if [ $CPU2_LEVEL -gt $CPU2_IDLE ]
	    then
		CPU2_TEST=$CPU2_LEVEL
            fi
	fi
    else
        touch /var/log/system.loaded
	CPU1_TEST=$CPU1_LEVEL
	CPU2_TEST=$CPU2_LEVEL
    fi
#  If we get to this part of the if statement /tmp/sensors.out
#  does not exist, the sensors failed somehow.
else
    CPU_LOG=-1
    CPU1_LOG=-1
    CPU2_LOG=-1
    CPU1_VAL=0
    CPU2_VAL=0
fi
#extra fi happens here so that the whole if only happens if
# we are not in test mode
fi
#
#   Check the conditions in increasing order of severity
#   Won't do this section in testing because CPU_LOG !=0 in that case
#
    if [ $CPU_LOG -eq 0 ] 
    then
	CPU1_THRESH_5=`expr $CPU1_TEST + $FIRSTLEV`
	CPU2_THRESH_5=`expr $CPU2_TEST + $FIRSTLEV`
	CPU1_THRESH_10=`expr $CPU1_TEST + $SECONDLEV`
	CPU2_THRESH_10=`expr $CPU2_TEST + $SECONDLEV`
# sometimes when dealing with a formerly idle system just
# starting up we will have to check the loaded threshold too.
        CPU1_LOAD_5=`expr $CPU1_LEVEL + $FIRSTLEV`
        CPU2_LOAD_5=`expr $CPU2_LEVEL + $FIRSTLEV`
        CPU1_LOAD_10=`expr $CPU1_LEVEL + $SECONDLEV`
        CPU2_LOAD_10=`expr $CPU2_LEVEL + $SECONDLEV`
#
#    5 degrees over threshhold
#
	if [ $CPU1_VAL -ge $CPU1_THRESH_5 ]
	then
	    CPU1_LOG=2
#
#    Deal with the case where cpu is busy but load is not yet 1:
#    2 possibilities:  Load idle, system under load threshold, first time--OK
#    Load idle, system already flagged as in transition, not under idle thresh.alarm
#
	    if [ "$IS_IDLE" == "YES" ] && [ "$WAS_INTRANS" == "NO" ] && [ "$WAS_TEMPALERT" == "NO" ]
	    then
		if [ $CPU1_VAL -lt $CPU1_LOAD_5 ] 
		then
		    CPU1_LOG=0
		    INTRANS="YES"
		    logger "CPU1 in possible idle-loaded transition, $CPU1_VAL, load $LOADNUM"
		    if [ ! -e /var/log/system.intrans ] 
		    then
			touch /var/log/system.intrans
                    fi
                fi
	    fi
	    if [ "$TEMPCPU1_5" == "YES" ] && [ $CPU1_LOG -eq 2 ] 
	    then
		CPU1_LOG=4
	    fi
	fi
	if [ $CPU2_VAL -ge $CPU2_THRESH_5 ] 
	then
	    CPU2_LOG=2
	    if [ "$IS_IDLE" == "YES" ] && [ "$WAS_INTRANS" == "NO" ] && [ "$WAS_TEMPALERT" == "NO" ]
	    then
		if [ $CPU2_VAL -lt $CPU2_LOAD_5 ] 
		then
		    CPU2_LOG=0
		    INTRANS="YES"
                    logger "CPU2 in possible idle-loaded transition, $CPU2_VAL, load $LOADNUM"
		    if [ ! -e /var/log/system.intrans ] 
		    then
			touch /var/log/system.intrans
		    fi
                fi
	    fi
	    if [ "$TEMPCPU2_5" == "YES" ] && [ $CPU2_LOG -eq 2 ]
	    then
		CPU2_LOG=4
	    fi
	fi
	if [ $CPU1_LOG -eq 2 ] || [ $CPU2_LOG -eq 2 ]
	then
	    CPU_LOG=2
        fi
	if [ $CPU1_LOG -eq 4 ] || [ $CPU2_LOG -eq 4 ]
	then
	    CPU_LOG=4
        fi
#
#    10 degrees over threshhold
#
	if [ $CPU1_VAL -ge $CPU1_THRESH_10 ]
	then
	    CPU1_LOG=3
	    if [ "$IS_IDLE" == "YES" ] && [ "$WAS_INTRANS" == "NO" ] && [ "$WAS_TEMPALERT" == "NO" ]
	    then
		if [ $CPU1_VAL -lt $CPU1_LOAD_10 ] 
		then
		    CPU1_LOG=0
		    INTRANS="YES"
		    logger "CPU1 in possible idle-loaded transition, $CPU1_VAL, load $LOADNUM"
		    if [ ! -e /var/log/system.intrans ]
		    then
			touch /var/log/system.intrans
		    fi
                fi
	    fi
	    if [ "$TEMPCPU1_10" == "YES" ] && [ $CPU1_LOG -eq 3 ]
	    then
		CPU1_LOG=5
	    fi
	fi
	if [ $CPU2_VAL -ge $CPU2_THRESH_10 ] 
	then
	    CPU2_LOG=3
	    if [ "$IS_IDLE" == "YES" ] && [ "$WAS_INTRANS" == "NO" ] && [ "$WAS_TEMPALERT" == "NO" ]
	    then
		if [ $CPU2_VAL -lt $CPU2_LOAD_10 ] 
		then
		    CPU2_LOG=0
		    INTRANS="YES"
		    logger "CPU2 in possible idle-loaded transition, $CPU2_VAL, load $LOADNUM"
		    if [ ! -e /var/log/system.intrans ]
		    then
			touch /var/log/system.intrans
		    fi
                fi
	    fi
	    if [ "$TEMPCPU2_10" == "YES" ] && [ $CPU2_LOG -eq 3 ]
	    then
		CPU2_LOG=5
	    fi
	fi
	if [ $CPU1_LOG -eq 3 ] || [ $CPU2_LOG -eq 3 ]
	then
	    if [ $CPU_LOG -lt 3 ]
	    then
		CPU_LOG=3
	    fi
	fi
	if [ $CPU1_LOG -eq 5 ] || [ $CPU2_LOG -eq 5 ]
	then
	    if [ $CPU_LOG -lt 5 ]
	    then
		CPU_LOG=5
	    fi
	fi
#
#       Over emergency threshhold
#
	if [ $CPU1_VAL -ge $CPU1_EMERG ]
	then
	    CPU1_LOG=6
	    if [ "$TEMPCPU1_EMERG" == "YES" ]
	    then
		CPU1_LOG=7
	    fi
	fi
	if [ $CPU2_VAL -ge $CPU2_EMERG ] 
	then
	    CPU2_LOG=6
	    if [ "$TEMPCPU2_EMERG" == "YES" ]
	    then
		CPU2_LOG=7
            fi
	fi
	if [ $CPU1_LOG -eq 6 ] || [ $CPU2_LOG -eq 6 ] 
 	then
	    CPU_LOG=6
	fi
	if [ $CPU1_LOG -eq 7 ] || [ $CPU2_LOG -eq 7 ]
	then
	    CPU_LOG=7
        fi
    fi
#  Handle the case of detecting jammed sensors correctly.
#  3 situations:
#  First detect of a possible jam, return cpu_log -1
#  and touch the sensors.jammed file
#  second jam situation, return cpu_log 1
#  "unavailable" but not jammed condition, but there is 
# a sensors_jammed file, return cpu_log 1
#
if [ $CPU1_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	CPU1_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "CPU1, sensors jammed, $CPU1_VAL"
	else
	    logger "CPU1, previous sensors jammed, now data unavailable, $CPU1_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "CPU1 First bad data, sensors possibly jammed, $CPU1_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "CPU1 First bad data, sensors possibly jammed, $CPU1_VAL" 
	    fi
	fi
    fi
fi
if [ $CPU2_LOG -eq -1 ] 
then
    if [ "$SENSORS_JAMMED" == "YES" ] 
    then
	CPU2_LOG=1
	if [ "$JAMMED" == "YES" ]
	then
	    logger "CPU2, sensors jammed, $CPU2_VAL"
	else
	    logger "CPU2, previous sensors jammed, now data unavailable, $CPU2_VAL"
	fi
    else
	if [ "$JAMMED" == "YES" ]
	then
	    if [ ! -s /var/log/sensors.jammed ]
	    then
		echo "CPU2 First bad data. sensors possibly jammed, $CPU2_VAL, `date`" > /var/log/sensors.jammed
		cat /tmp/sensors.out >> /var/log/sensors.jammed
		logger "CPU2 First bad data, sensors possibly jammed, $CPU1_VAL" 
	    fi
	fi
    fi
fi
if [ $CPU1_LOG -eq -1 ] || [ $CPU2_LOG -eq -1 ] 
then
    CPU_LOG=-1
fi
if [ $CPU1_LOG -eq 1 ] || [ $CPU2_LOG -eq 1 ] 
then
    CPU_LOG=1
fi
#
# fix the record files for history
#
case $CPU1_LOG in 
    0) 
    if [ -s /var/log/tempcpu1_5 ]
    then
	rm /var/log/tempcpu1_5 
    fi
    if [ -s /var/log/tempcpu1_10 ]
    then
	rm /var/log/tempcpu1_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu1_emerg ]
    then
	rm /var/log/tempcpu1_emerg
    fi
    ;;
    1)
    if [ -s /var/log/tempcpu1_5 ]
    then
	rm /var/log/tempcpu1_5 
    fi
    if [ -s /var/log/tempcpu1_10 ]
    then
	rm /var/log/tempcpu1_10
    fi
    if [ -s /var/log/tempcpu1_emerg ]
    then
	rm /var/log/tempcpu1_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    2) 
    if [ ! -s /var/log/tempcpu1_5 ]
    then
	echo "CPU1 $FIRSTLEV degrees over temp, $CPU1_VAL, `date`" > /var/log/tempcpu1_5
	cat /tmp/sensors.out >> /var/log/tempcpu1_5
	logger "CPU1 at least $FIRSTLEV degrees over temp, $CPU1_VAL, load $LOADNUM"
    fi
    if [ -s /var/log/tempcpu1_10 ]
    then
	rm /var/log/tempcpu1_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu1_emerg ]
    then
	rm /var/log/tempcpu1_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    3) 
    if [ ! -s /var/log/tempcpu1_10 ]
    then
	echo "CPU1 $SECONDLEV degrees over temp, $CPU1_VAL, `date`" > /var/log/tempcpu1_10
	cat /tmp/sensors.out >> /var/log/tempcpu1_10
	logger "CPU1 at least $SECONDLEV degrees over temp, $CPU1_VAL, load $LOADNUM"
    fi
    if [ -s /var/log/tempcpu1_5 ]
    then
	rm /var/log/tempcpu1_5
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu1_emerg ]
    then
	rm /var/log/tempcpu1_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    4) 
    if [ -s /var/log/tempcpu1_10 ] 
    then
	rm /var/log/tempcpu1_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "CPU1 continued at least $FIRSTLEV degrees over temp, $CPU1_VAL, load $LOADNUM"
    if [ -s /var/log/tempcpu1_emerg ]
    then
	rm /var/log/tempcpu1_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    5) 
    if [ -s /var/log/tempcpu1_5 ] 
    then
	rm /var/log/tempcpu1_5
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    logger "CPU1 continued at least 10 degrees over temp, $CPU1_VAL, load $LOADNUM"
    if [ -s /var/log/tempcpu1_emerg ]
    then
	rm /var/log/tempcpu1_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    6)
    if [ -s /var/log/tempcpu1_5 ]
    then
	rm /var/log/tempcpu1_5
    fi
    if [ -s /var/log/tempcpu1_10 ]
    then
	rm /var/log/tempcpu1_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ ! -s /var/log/tempcpu1_emerg ]
    then
	echo "CPU1 over emergency temp, $CPU1_VAL, `date`" > /var/log/tempcpu1_emerg
	cat /tmp/sensors.out >> /var/log/tempcpu1_emerg
        date >> /var/log/tempprob.log
        cat /tmp/sensors.out >> /var/log/tempprob.log
    fi
    logger "CPU1 over emergency temperature, $CPU1_VAL, load $LOADNUM"
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    7)
    if [ -s /var/log/tempcpu1_5 ]
    then
	rm /var/log/tempcpu1_5
    fi
    if [ -s /var/log/tempcpu1_10 ]
    then
	rm /var/log/tempcpu1_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi 
    logger "CPU1 continued over emergency temperature, $CPU1_VAL, load $LOADNUM"
            date >> /var/log/tempprob.log
	    cat /tmp/sensors.out >> /var/log/tempprob.log
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
   ;;
esac
case $CPU2_LOG in 
    0) 
    if [ -s /var/log/tempcpu2_5 ]
    then
	rm /var/log/tempcpu2_5 
    fi
    if [ -s /var/log/tempcpu2_10 ]
    then
	rm /var/log/tempcpu2_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu2_emerg ]
    then
	rm /var/log/tempcpu2_emerg
    fi
    ;;
    1)
    if [ -s /var/log/tempcpu2_5 ]
    then
	rm /var/log/tempcpu2_5 
    fi
    if [ -s /var/log/tempcpu2_10 ]
    then
	rm /var/log/tempcpu2_10
    fi
    if [ -s /var/log/tempcpu2_emerg ]
    then
	rm /var/log/tempcpu2_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    2) 
    if [ ! -s /var/log/tempcpu2_5 ]
    then
	echo "CPU2 $FIRSTLEV degrees over temp, $CPU2_VAL, `date`" > /var/log/tempcpu2_5
	cat /tmp/sensors.out >> /var/log/tempcpu2_5
	logger "CPU2 at least $FIRSTLEV degrees over temp, $CPU2_VAL, load $LOADNUM"
    fi
    if [ -s /var/log/tempcpu2_10 ]
    then
	rm /var/log/tempcpu2_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu2_emerg ]
    then
	rm /var/log/tempcpu2_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    3) 
    if [ ! -s /var/log/tempcpu2_10 ]
    then
	echo "CPU2 $SECONDLEV degrees over temp, $CPU2_VAL, `date`" > /var/log/tempcpu2_10
	cat /tmp/sensors.out >> /var/log/tempcpu2_10
	logger "CPU2 at least $SECONDLEV degrees over temp, $CPU2_VAL, load $LOADNUM"
    fi
    if [ -s /var/log/tempcpu2_5 ]
    then
	rm /var/log/tempcpu2_5
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu2_emerg ]
    then
	rm /var/log/tempcpu2_emerg
    fi
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    4) 
    if [ -s /var/log/tempcpu2_10 ] 
    then
	rm /var/log/tempcpu2_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu2_emerg ]
    then
	rm /var/log/tempcpu2_emerg
    fi
    logger "CPU2 continued at least $FIRSTLEV degrees over temp, $CPU2_VAL, load $LOADNUM"
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    5) 
    if [ -s /var/log/tempcpu2_5 ] 
    then
	rm /var/log/tempcpu2_5
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ -s /var/log/tempcpu2_emerg ]
    then
	rm /var/log/tempcpu2_emerg
    fi
    logger "CPU2 continued at least $SECONDLEV degrees over temp, $CPU2_VAL, load $LOADNUM"
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    6)
    if [ -s /var/log/tempcpu2_5 ]
    then
	rm /var/log/tempcpu2_5
    fi
    if [ -s /var/log/tempcpu2_10 ]
    then
	rm /var/log/tempcpu2_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi
    if [ ! -s /var/log/tempcpu2_emerg ]
    then
	echo "CPU2 over emergency temp, $CPU2_VAL, `date`" > /var/log/tempcpu2_emerg
	cat /tmp/sensors.out >> /var/log/tempcpu2_emerg
            date >> /var/log/tempprob.log
	    cat /tmp/sensors.out >> /var/log/tempprob.log

    fi
    logger "CPU2 over emergency temperature, $CPU2_VAL, load $LOADNUM"
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
    ;;
    7)
    if [ -s /var/log/tempcpu2_5 ]
    then
	rm /var/log/tempcpu2_5
    fi
    if [ -s /var/log/tempcpu2_10 ]
    then
	rm /var/log/tempcpu2_10
    fi
    if [ -s /var/log/sensors.jammed ]
    then
	rm /var/log/sensors.jammed
    fi 
    logger "CPU2 continued over emergency temperature, $CPU2_VAL, load $LOADNUM"
            date >> /var/log/tempprob.log
	    cat /tmp/sensors.out >> /var/log/tempprob.log
    if [ -e /var/log/system.intrans ]
    then
	rm /var/log/system.intrans
    fi
   ;;
esac
# Output section
if [ "$LOGRES" == "LOGICAL" ]
then
    case $NCPU in
    0) 
	echo $CPU_LOG
    ;;
    1) 
	echo $CPU1_LOG
    ;;
    2) 
	echo $CPU2_LOG
    ;;
    *)
    echo "invalid argument to -n"
	echo $CPU_LOG
    ;;
    esac
else
    case $NCPU in 
    0)
	echo "$CPU1_VAL $CPU2_VAL" | sed -e 's/+//g'
    ;;
    1) 
	echo $CPU1_VAL | sed -e 's/+//g'
    ;;
    2)
	echo $CPU2_VAL  | sed -e 's/+//g'
    ;; 
    *) 
    echo "invalid argument to -n"
	echo "$CPU1_VAL $CPU2_VAL"
    ;;
    esac
fi
#
#  Run Ganglia section on only the all-cpu iteration of 
#  this script (NCPU=0)...normally the ngop script will run it 3 times
#  in quick succession with sensors_wrap -l ; sensors_wrap -l -n 1 ;
#   sensors_wrap -l -n 2 
#
if [ $NCPU -eq 0 ] 
then
#
#  Ganglia section
#  Now broadcast the values to ganglia using the "gmetric" tool
#  modified from a script by Mike Snitzer
#
GMETRIC_BIN=/usr/bin/gmetric
# Get the multicast channel to send out to:
#
#   In Fermi Farms config mcast_channel line is often commented out
#   use the default mcast_channel if so (or if /etc/gmond.conf doesn't exist)
MCASTCHAN=""
if [ -e /etc/gmond.conf ]
then
    MCASTCHAN="`egrep ^mcast_channel /etc/gmond.conf | awk '{print $2}'`"
    if [ "$MCASTCHAN" == "" ]
    then
	GMETRIC="${GMETRIC_BIN} "
    else
	GMETRIC="${GMETRIC_BIN} -c ${MCASTCHAN} "
    fi
fi
	
#   establish a base commandline
#
#   check to be sure gmond is running
#
`/sbin/service gmond status > /dev/null`
if [ $? -eq 0 ] && [ -x $GMETRIC_BIN ] 
then
#
# send cpu temperatures
#
    $GMETRIC  --type int16 --name "temp_cpu1" --units "Celsius" --value $CPU1_VAL
    $GMETRIC  --type int16 --name "temp_cpu2" --units "Celsius" --value $CPU2_VAL
    $GMETRIC  --type int16 --name "cputemp_ok" --units "binary" --value $CPU_LOG
fi
fi
